/*
 *	ccryptgui, QT based graphical user interface to ccrypt
 *
 *	Copyright (C) 2011  Martin Hilscher
 *
 *	This program is free software; you can redistribute it and/or modify it under the
 *	terms of the GNU General Public License as published by theFree Software Foundation;
 *	either version 3 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *	PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along with this
 *	program; if not, see <http://www.gnu.org/licenses/>.
 *
 *	----------------------------------------------------------------------
 *
 *	ccryptgui.cpp -  defining base-class functions	
 */


#include "ccryptgui.h"

#ifdef WIN
   #include "direct.h"
   #include "process.h"
   #include "winreg.h"
#else //LINUX
   #include "unistd.h"
   #include "sys/wait.h"
   #include "sys/stat.h"
#endif


//==============
// Constructor
//==============
  
  CCryptGui::CCryptGui( )
  {

     // Provide Gui-windows
     mainwindow  = new QMainWindow;
     prefwindow  = new QDialog;
     pwdwindow   = new QDialog;
     aboutwindow = new QDialog;
     filedialog  = new QFileDialog;
     
     // Initializing of Gui-windows
     ui_main  = new Ui_MainWindow;
     ui_pref  = new Ui_PrefWindow;
     ui_pwd   = new Ui_PwdWindow;
     ui_about = new Ui_AboutWindow;
     ui_main->setupUi(mainwindow);
     ui_pref->setupUi(prefwindow);
     ui_pwd->setupUi(pwdwindow);
     ui_about->setupUi(aboutwindow);

     // Set path of ini-files and temp-files
#ifdef WIN
     init_path  = getenv ( "APPDATA" );
     init_path += "\\ccryptgui";
     _mkdir ( init_path.data() );
#else
     init_path = getenv ( "HOME" );
#endif
     temp_path  = init_path;
     temp_path += DIR_SLASH;
     temp_path += TMP_DIR_NAME;

     // Read preferences from ini file
     get_ini();

     // Miscellanious initializations
     enable_log = 0;

     // Set widgets of preference-window
     set_pref_widgets();  

     // Set fonts
     set_fonts();

     // Set version and copyright of About-Dialog
     ui_about->label_V->setText( QString( "Version: " ) + VERSION + " - " + VERSIONDATE );
     ui_about->label_C->setText( QString( COPYRIGHT ) );

     // Initialize filedialog
     filedialog->setFileMode(QFileDialog::ExistingFiles);
     filedialog->setViewMode(QFileDialog::Detail);
     ui_main->checkBox->setCheckState(Qt::Unchecked);
     filedialog_mode = FILE_DIALOG_FILES_MODE;

     filedialog->setDirectory( curr_path );


     // Connect signals
     //===================

     // Mainwindow
     //--------------

     // CLose all windows, if mainwindow will be closed
     connect(mainwindow, SIGNAL(destroyed()), qApp, SLOT(closeAllWindows()));

     // Menubar
     connect(ui_main->actionClose, SIGNAL(activated()), qApp, SLOT(quit()));
     connect(ui_main->actionClose, SIGNAL(activated()), this, SLOT(remove_temp_dir()));
     connect(ui_main->action_Preferences, SIGNAL(activated()), prefwindow, SLOT(show()));
     connect(ui_main->action_Preferences, SIGNAL(activated()), this, SLOT(disable_buttons()));
//     connect(ui_main->actionHelp, SIGNAL(activated()), helpwindow, SLOT(show()));
     connect(ui_main->actionHelp, SIGNAL(activated()), this, SLOT(call_help()));
     connect(ui_main->actionAbout, SIGNAL(activated()), aboutwindow, SLOT(show()));
     connect(ui_main->actionAboutQt, SIGNAL(activated()), qApp, SLOT(aboutQt()));

     // En-, Decrypt and Open-buttons / checkbox for open-mode
     connect(ui_main->pushButtonE, SIGNAL(clicked()), this, SLOT(encrypt()));
     connect(ui_main->pushButtonD, SIGNAL(clicked()), this, SLOT(decrypt()));
     connect(ui_main->pushButtonO, SIGNAL(clicked()), this, SLOT(open()));
     connect(ui_main->checkBox, SIGNAL(stateChanged(int)), this, SLOT(open_mode()));


     // Filedialog
     //-------------

     // OK-button / Cancel-button
     connect(filedialog, SIGNAL(accepted()), this, SLOT(start_crypt()));
     connect(filedialog, SIGNAL(rejected()), this, SLOT(enable_buttons()));


     // Passwort-window
     //------------------

     // OK-button / Abort-button
     connect(pwdwindow, SIGNAL(accepted()), this, SLOT(password_Ok()));
     connect(pwdwindow, SIGNAL(rejected()), this, SLOT(password_Abort()));
    

     // Preference-window
     //---------------------
     connect(prefwindow, SIGNAL(rejected()), this, SLOT(enable_buttons()));

     // Default-, Cancel-, Apply- und OK-buttons
     connect(ui_pref->buttonDefault, SIGNAL(clicked()), this, SLOT(set_default()));
     connect(ui_pref->buttonCancel, SIGNAL(clicked()), prefwindow, SLOT(close()));
     connect(ui_pref->buttonCancel, SIGNAL(clicked()), this, SLOT(enable_buttons()));
     connect(ui_pref->buttonApply, SIGNAL(clicked()), this, SLOT(change_pref()));
     connect(ui_pref->buttonOk, SIGNAL(clicked()), prefwindow, SLOT(close()));
     connect(ui_pref->buttonOk, SIGNAL(clicked()), this, SLOT(change_pref()));
     connect(ui_pref->buttonOk, SIGNAL(clicked()), this, SLOT(enable_buttons()));

     // Fonts
     connect(ui_pref->spinBox,   SIGNAL(valueChanged(int)), this, SLOT(set_fonts()));
     connect(ui_pref->spinBox_2, SIGNAL(valueChanged(int)), this, SLOT(set_fonts()));


     // About-window
     //-----------------
     connect(ui_about->pushButton, SIGNAL(clicked()), this, SLOT(call_homepage()));
  }


//===================
// Mainwindow
//===================
    
  // Encrypt
  void CCryptGui::encrypt( )
  {

     if ( filedialog_mode == FILE_DIALOG_DIR_MODE ) filedialog->setFileMode(QFileDialog::Directory);
     else                                           filedialog->setFileMode(QFileDialog::ExistingFiles);

     QStringList filters;
     filters << "All Files (*)" << "Crypt-Files (*.cpt)";
     filedialog->setFilters( filters );
     ui_main->textEdit->clear();
     ui_main->progressBar->setValue(0);
     filedialog->show();
     crypt_mode = CRYPT_MODE_ENCRYPT;
     disable_buttons();


     // Debug mode
     if ( debug_mode == DEBUG_MODE )
     {
        printf ("\n");
        printf ("Encrypt\n");
        debug_infos( );
     }
  }

  // Decrypt
  void CCryptGui::decrypt( )
  {

     if ( filedialog_mode == FILE_DIALOG_DIR_MODE ) filedialog->setFileMode(QFileDialog::Directory);
     else                                           filedialog->setFileMode(QFileDialog::ExistingFiles);

     QStringList filters;
     filters << "Crypt-Files (*.cpt)" << "All Files (*)";
     filedialog->setFilters( filters );
     ui_main->textEdit->clear();
     ui_main->progressBar->setValue(0);
     filedialog->show();
     crypt_mode = CRYPT_MODE_DECRYPT;
     disable_buttons();


     // Debug mode
     if ( debug_mode == DEBUG_MODE )
     {
        printf ("\n");
        printf ("Decrypt\n");
        debug_infos( );
     }

  }

  // Open
  void CCryptGui::open( )
  {

     filedialog->setFileMode(QFileDialog::ExistingFiles);

     QStringList filters;
     filters << "Crypt-Files (*.cpt)" << "All Files (*)";
     filedialog->setFilters( filters );
     ui_main->textEdit->clear();
     ui_main->progressBar->setValue( 0 );
     filedialog->show();
     crypt_mode = CRYPT_MODE_OPEN;
     disable_buttons();


     // Debug mode
     if ( debug_mode == DEBUG_MODE )
     {
        printf ("\n");
        printf ("Open\n");
        debug_infos( );
     }

  }

  // Change direktory-mode of filedialog
  void CCryptGui::open_mode( )
  {
     if (ui_main->checkBox->checkState() == Qt::Checked)
     {
        filedialog_mode = FILE_DIALOG_DIR_MODE;
     }
     else
     {
        filedialog_mode = FILE_DIALOG_FILES_MODE;
     }


     // Debug mode
     if ( debug_mode == DEBUG_MODE )
     {
        printf ("\n");
        printf ("Open Mode\n");
        debug_infos( );
     }

  }


  // Remove temp-dir
  void CCryptGui::remove_temp_dir ( )
  {
     QByteArray temp;
     int status;

#ifdef WIN

     temp  = "rmdir /S /Q \"";
     temp += temp_path.data();
     temp += "\"";
     status = system( temp );

     if ( status < 0 )
     {
        temp  = "** ERROR: remove_temp_dir: call rmdir: ERR = ";
        temp += QByteArray::number( status );
        write_messages( temp.data() );
     }

#else //Linux

     pid_t pid;

     switch ( pid = fork() )
     {
     // fork-error
     case -1:

        write_messages( "** ERROR: remove_temp_dir: could not call child-process" );

     // child
     case 0:

        execlp( "rm", "rm", "-r", temp_path.data(), NULL );
        exit(127);

     // parents
     default:

        pid = wait( &status );

        if ( status )
        {
           temp  = "** ERROR: remove_temp_dir: call process: ERR = ";
           temp += QByteArray::number( status );
           write_messages( temp.data() );
        } 
     }

#endif

  }



//=====================
// Preference-Window
//=====================
    
  // Change preferences
  void CCryptGui::change_pref()
  {

     cmd_encrypt = (ui_pref->lineEditE->text()).toAscii();

     cmd_decrypt = (ui_pref->lineEditD->text()).toAscii();

     log_file_name = (ui_pref->lineEdit_log->text()).toAscii();

     if ( ui_pref->radioButton_log->isChecked() )
        enable_log_file = 1;
     else
        enable_log_file = 0;

     save_ini();

     // Set fonts
     // set_fonts();

     // Set argument list of "ccrypt"
     set_cmd_args();

     // Debug mode
     if ( debug_mode == DEBUG_MODE )
     {
        printf ("\n");
        printf ("Change preferences\n");
        debug_infos( );
     }

  }


  // Set default values
  void CCryptGui::set_default()
  {

     log_file_name = DEFAULT_LOG_FILE_NAME;
     cmd_encrypt = DEFAULT_CMD_ENCRYPT;
     cmd_decrypt = DEFAULT_CMD_DECRYPT;
     enable_log_file = DEFAULT_ENABLE_LOG_FILE;
     font_size = DEFAULT_FONT_SIZE;
     font_size_header = DEFAULT_FONT_SIZE_HEADER;

#ifdef WIN

     // read registry

     LONG ret;
     HKEY hKey;
     BYTE str[256];
     char * pos1;
     char * pos2;

     DWORD size = sizeof(str);

     RegOpenKeyExA( HKEY_CURRENT_USER, SUBKEY_EIGENE_DATEIN, NULL, KEY_QUERY_VALUE, &hKey);
     ret = RegQueryValueExA(hKey, KEY_VALUE_EIGENE_DATEIN, NULL, NULL, str, &size);
     RegCloseKey(hKey);

     if (ret != ERROR_SUCCESS)
     {
        curr_path = getenv( "USERPROFILE");
        ui_main->textEdit->setText( "** ERROR: Path \"Eigene Dateien / My Documents \" not found in Registry!" ) ;
     }
     else
     {
         // extract and substitude environment variables in path string
         // it will be assumed, that the environment variable is at the
         // beginning of the the path e.g. "%<env>%\path\...
         pos1 = strtok( (char*)str , "%" );
         pos2 = strtok( NULL , "%" );
         if ( ( pos1 > (char*)str ) || ( pos2 != NULL ) )
         {
            if ( pos2 )
            {
               if ( pos1 == ( (char*)str + 1 ) )
               {
                   curr_path  = getenv( pos1 );
                   curr_path += pos2;
               }
               else
               {
                   curr_path = getenv( "USERPROFILE");
                   ui_main->textEdit->setText( "** ERROR: Path \"Eigene Dateien / My Documents \" corrupted in Registry!" ) ;
               }
            }
            else
            {
                curr_path = getenv( "USERPROFILE");
                ui_main->textEdit->setText( "** ERROR: Path \"Eigene Dateien / My Documents \" corrupted in Registry!" ) ;
            }
         }
         else
            curr_path = (char*)str;
     }

#else
     curr_path = init_path;
#endif

     filedialog->setDirectory( curr_path );

     // Set widgets of the preference-window
     set_pref_widgets();


     // Debug mode
     if ( debug_mode == DEBUG_MODE )
     {
        printf ("\n");
        printf ("Set Defaults\n");
        debug_infos( );
     }

  }



//=====================
// Filedialog-Window
//=====================

  // Start converting
  void CCryptGui::start_crypt( )
  {

     char  str[30];
     int   cnt_cpt;
     QDir  dir;
     QByteArray temp;
 

     // Initialize log-file

     enable_log = 0;
 
     if ( enable_log_file )
     {
        init_logfile ( );
     }

     filelist  = filedialog->selectedFiles();
     n_files   = filelist.size();
 
     dir       = filedialog->directory();
     curr_path = (dir.absolutePath()).toAscii();

     save_ini();

     if ( crypt_mode == CRYPT_MODE_OPEN )
     {
         // test if temp_path exists
         if ( chdir( temp_path ) )
        {
#ifdef WIN
           if ( mkdir( temp_path ) )
#else
           if ( mkdir( temp_path, S_IRWXU ) )
#endif
           {
              write_messages( "** ERROR: could not create temp directory" );
              enable_buttons();
              return;
           }
           else
           {
              write_messages( "** INFO: temp directory created" );
           }
        }
        else
        {
           write_messages( "** INFO: temp directory already exits" );
        }
        write_messages( "**       please take care, that it will be complete deleted" );
        write_messages( "**       please close \"ccryptgui\" with File->Close" );

        for (cnt_cpt = 0; cnt_cpt < n_files; cnt_cpt++)
        {
           if ( cpy_files_to_tmp ( cnt_cpt ) )
           {
              write_messages( "** ERROR: could not copy files to temp-directory" );
              enable_buttons();
              return;
           }
        }
     }


     if ( crypt_mode == CRYPT_MODE_ENCRYPT )
     {
        sprintf( str, "Encrypt %i ", n_files);
        ui_pwd->label_2->setEnabled(true);
        ui_pwd->label1->setEnabled(true);
        ui_pwd->label12->setEnabled(true);
        ui_pwd->lineEditC->setEnabled(true);
     }
     else // (( crypt_mode == CRYPT_MODE_DECRYPT ) || ( crypt_mode == CRYPT_MODE_OPEN ))
     {
        sprintf( str, "Decrypt %i ", n_files);
        ui_pwd->label_2->setEnabled(false);
        ui_pwd->label1->setEnabled(false);
        ui_pwd->label12->setEnabled(false);
        ui_pwd->lineEditC->setEnabled(false);
     }

     if ( filedialog_mode == FILE_DIALOG_FILES_MODE )
        strcat( str, "File(s):");
     else
        strcat( str, "Directory:");

     write_messages( str );


     ui_main->progressBar->setMaximum(n_files + 1);
     ui_main->progressBar->setValue(1);

     pwdwindow->show();


     // Debug mode

     if ( debug_mode == DEBUG_MODE )
     {
        printf ("\n");
        printf ("Start Crypt\n");
        debug_infos( );
        for (int j=0; j<=n_files+1; j++)
        {
           temp = (filelist.value(j)).toAscii();
           printf("filelist: %s\n", temp.data() );
        }
     }

  }

//=========================
// Passwort-Dialog-Window
//=========================

  // OK-button pressed
  void CCryptGui::password_Ok()
  {

     QByteArray temp;


     // Execute crypt
     crypt_exe ( );


     // Open application
     if ( crypt_mode == CRYPT_MODE_OPEN )
     {
        temp  = "** ";
        temp += QByteArray::number( execute_appl ( ) );
        temp += " File(s) opened";
        write_messages( temp.data() );

//        remove_temp_dir( );    ( the content of some applications would disappear )
     }


     // Copy messages to log-file and close it
     if ( enable_log )
     {
        close_logfile( );
     }


     // Debug mode
     if ( debug_mode == DEBUG_MODE )
     {
        printf ("\n");
        printf ("Password OK\n");
        debug_infos( );
     }


     enable_buttons();

     return;

  }


  // Abort-button pressed
  void CCryptGui::password_Abort()
  {
     enable_buttons();
     ui_main->progressBar->setValue(0);

     write_messages( "** abort by user" );

     if ( crypt_mode == CRYPT_MODE_OPEN )
     {
        remove_temp_dir( );
     }
     if ( enable_log )
     {
        close_logfile( );
     }
  }




